@@ -2,10 +2,7 @@
 /*
  * soup-gnutls.c
  *
- * Authors:
- *      Ian Peters <itp@ximian.com>
- *
- * Copyright (C) 2003, Ximian, Inc.
+ * Copyright (C) 2003-2006, Novell, Inc.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -15,6 +12,7 @@
 #ifdef HAVE_SSL
 
 #include <errno.h>
+#include <fcntl.h>
 #include <pthread.h>
 #include <stdlib.h>
 #include <string.h>
@@ -32,17 +30,17 @@ gboolean soup_ssl_supported = TRUE;
 
 #define DH_BITS 1024
 
-typedef struct {
-	gnutls_certificate_credentials cred;
+struct SoupSSLCredentials {
+	gnutls_certificate_credentials creds;
 	gboolean have_ca_file;
-} SoupGNUTLSCred;
+};
 
 typedef struct {
 	GIOChannel channel;
 	int fd;
 	GIOChannel *real_sock;
 	gnutls_session session;
-	SoupGNUTLSCred *cred;
+	SoupSSLCredentials *creds;
 	char *hostname;
 	gboolean established;
 	SoupSSLType type;
@@ -129,21 +127,26 @@ verify_certificate (gnutls_session session, const char *hostname, GError **err)
 	return TRUE;
 }
 
+#define SOUP_GNUTLS_CHANNEL_NONBLOCKING(chan) (fcntl ((chan)->fd, F_GETFL, 0) & O_NONBLOCK)
+
 static GIOStatus
 do_handshake (SoupGNUTLSChannel *chan, GError **err)
 {
 	int result;
 
+again:
 	result = gnutls_handshake (chan->session);
 
-	if (result == GNUTLS_E_AGAIN ||
-	    result == GNUTLS_E_INTERRUPTED) {
-		g_set_error (err, SOUP_SSL_ERROR,
-			     (gnutls_record_get_direction (chan->session) ?
-			      SOUP_SSL_ERROR_HANDSHAKE_NEEDS_WRITE :
-			      SOUP_SSL_ERROR_HANDSHAKE_NEEDS_READ),
-			     "Handshaking...");
-		return G_IO_STATUS_AGAIN;
+	if (result == GNUTLS_E_AGAIN || result == GNUTLS_E_INTERRUPTED) {
+		if (SOUP_GNUTLS_CHANNEL_NONBLOCKING (chan)) {
+			g_set_error (err, SOUP_SSL_ERROR,
+				     (gnutls_record_get_direction (chan->session) ?
+				      SOUP_SSL_ERROR_HANDSHAKE_NEEDS_WRITE :
+				      SOUP_SSL_ERROR_HANDSHAKE_NEEDS_READ),
+				     "Handshaking...");
+			return G_IO_STATUS_AGAIN;
+		} else
+			goto again;
 	}
 
 	if (result < 0) {
@@ -153,7 +156,7 @@ do_handshake (SoupGNUTLSChannel *chan, GError **err)
 		return G_IO_STATUS_ERROR;
 	}
 
-	if (chan->type == SOUP_SSL_TYPE_CLIENT && chan->cred->have_ca_file &&
+	if (chan->type == SOUP_SSL_TYPE_CLIENT && chan->creds->have_ca_file &&
 	    !verify_certificate (chan->session, chan->hostname, err))
 		return G_IO_STATUS_ERROR;
 
@@ -172,6 +175,7 @@ soup_gnutls_read (GIOChannel   *channel,
 
 	*bytes_read = 0;
 
+again:
 	if (!chan->established) {
 		result = do_handshake (chan, err);
 
@@ -186,13 +190,17 @@ soup_gnutls_read (GIOChannel   *channel,
 
 	if (result == GNUTLS_E_REHANDSHAKE) {
 		chan->established = FALSE;
-		return G_IO_STATUS_AGAIN;
+		goto again;
+	}
+
+	if (result == GNUTLS_E_INTERRUPTED || result == GNUTLS_E_AGAIN) {
+		if (SOUP_GNUTLS_CHANNEL_NONBLOCKING (chan))
+			return G_IO_STATUS_AGAIN;
+		else
+			goto again;
 	}
 
 	if (result < 0) {
-		if ((result == GNUTLS_E_INTERRUPTED) ||
-		    (result == GNUTLS_E_AGAIN))
-			return G_IO_STATUS_AGAIN;
 		g_set_error (err, G_IO_CHANNEL_ERROR,
 			     G_IO_CHANNEL_ERROR_FAILED,
 			     "Received corrupted data");
@@ -216,6 +224,7 @@ soup_gnutls_write (GIOChannel   *channel,
 
 	*bytes_written = 0;
 
+again:
 	if (!chan->established) {
 		result = do_handshake (chan, err);
 
@@ -228,15 +237,22 @@ soup_gnutls_write (GIOChannel   *channel,
 
 	result = gnutls_record_send (chan->session, buf, count);
 
+	/* I'm pretty sure this can't actually happen in response to a
+	 * write, but...
+	 */
 	if (result == GNUTLS_E_REHANDSHAKE) {
 		chan->established = FALSE;
-		return G_IO_STATUS_AGAIN;
+		goto again;
+	}
+
+	if (result == GNUTLS_E_INTERRUPTED || result == GNUTLS_E_AGAIN) {
+		if (SOUP_GNUTLS_CHANNEL_NONBLOCKING (chan))
+			return G_IO_STATUS_AGAIN;
+		else
+			goto again;
 	}
 
 	if (result < 0) {
-		if ((result == GNUTLS_E_INTERRUPTED) ||
-		    (result == GNUTLS_E_AGAIN))
-			return G_IO_STATUS_AGAIN;
 		g_set_error (err, G_IO_CHANNEL_ERROR,
 			     G_IO_CHANNEL_ERROR_FAILED,
 			     "Received corrupted data");
@@ -352,7 +368,7 @@ THROW_CREATE_ERROR:
  * @sock: a #GIOChannel wrapping a TCP socket.
  * @type: whether this is a client or server socket
  * @remote_host: the hostname of the remote machine
- * @credentials: a client or server credentials structure
+ * @creds: a client or server credentials structure
  *
  * This attempts to wrap a new #GIOChannel around @sock that
  * will SSL-encrypt/decrypt all traffic through it.
@@ -362,17 +378,16 @@ THROW_CREATE_ERROR:
  **/
 GIOChannel *
 soup_ssl_wrap_iochannel (GIOChannel *sock, SoupSSLType type,
-			 const char *remote_host, gpointer credentials)
+			 const char *remote_host, SoupSSLCredentials *creds)
 {
 	SoupGNUTLSChannel *chan = NULL;
 	GIOChannel *gchan = NULL;
 	gnutls_session session = NULL;
-	SoupGNUTLSCred *cred = credentials;
 	int sockfd;
 	int ret;
 
 	g_return_val_if_fail (sock != NULL, NULL);
-	g_return_val_if_fail (credentials != NULL, NULL);
+	g_return_val_if_fail (creds != NULL, NULL);
 
 	sockfd = g_io_channel_unix_get_fd (sock);
 	if (!sockfd) {
@@ -389,7 +404,7 @@ soup_ssl_wrap_iochannel (GIOChannel *sock, SoupSSLType type,
 		goto THROW_CREATE_ERROR;
 
 	if (gnutls_credentials_set (session, GNUTLS_CRD_CERTIFICATE,
-				    cred->cred) != 0)
+				    creds->creds) != 0)
 		goto THROW_CREATE_ERROR;
 
 	if (type == SOUP_SSL_TYPE_SERVER)
@@ -401,7 +416,7 @@ soup_ssl_wrap_iochannel (GIOChannel *sock, SoupSSLType type,
 	chan->fd = sockfd;
 	chan->real_sock = sock;
 	chan->session = session;
-	chan->cred = cred;
+	chan->creds = creds;
 	chan->hostname = g_strdup (remote_host);
 	chan->type = type;
 	g_io_channel_ref (sock);
@@ -451,22 +466,22 @@ soup_gnutls_init (void)
  * Return value: the client credentials, which must be freed with
  * soup_ssl_free_client_credentials().
  **/
-gpointer
+SoupSSLCredentials *
 soup_ssl_get_client_credentials (const char *ca_file)
 {
-	SoupGNUTLSCred *cred;
+	SoupSSLCredentials *creds;
 	int status;
 
 	if (!soup_gnutls_inited)
 		soup_gnutls_init ();
 
-	cred = g_new0 (SoupGNUTLSCred, 1);
-	gnutls_certificate_allocate_credentials (&cred->cred);
+	creds = g_new0 (SoupSSLCredentials, 1);
+	gnutls_certificate_allocate_credentials (&creds->creds);
 
 	if (ca_file) {
-		cred->have_ca_file = TRUE;
+		creds->have_ca_file = TRUE;
 		status = gnutls_certificate_set_x509_trust_file (
-			cred->cred, ca_file, GNUTLS_X509_FMT_PEM);
+			creds->creds, ca_file, GNUTLS_X509_FMT_PEM);
 		if (status < 0) {
 			g_warning ("Failed to set SSL trust file (%s).",
 				   ca_file);
@@ -478,7 +493,7 @@ soup_ssl_get_client_credentials (const char *ca_file)
 		}
 	}
 
-	return cred;
+	return creds;
 }
 
 /**
@@ -486,15 +501,13 @@ soup_ssl_get_client_credentials (const char *ca_file)
  * @creds: a client credentials structure returned by
  * soup_ssl_get_client_credentials().
  *
- * Frees @client_creds.
+ * Frees @creds.
  **/
 void
-soup_ssl_free_client_credentials (gpointer creds)
+soup_ssl_free_client_credentials (SoupSSLCredentials *creds)
 {
-	SoupGNUTLSCred *cred = creds;
-
-	gnutls_certificate_free_credentials (cred->cred);
-	g_free (cred);
+	gnutls_certificate_free_credentials (creds->creds);
+	g_free (creds);
 }
 
 /**
@@ -510,10 +523,10 @@ soup_ssl_free_client_credentials (gpointer creds)
  * Return value: the server credentials, which must be freed with
  * soup_ssl_free_server_credentials().
  **/
-gpointer
+SoupSSLCredentials *
 soup_ssl_get_server_credentials (const char *cert_file, const char *key_file)
 {
-	SoupGNUTLSCred *cred;
+	SoupSSLCredentials *creds;
 
 	if (!soup_gnutls_inited)
 		soup_gnutls_init ();
@@ -522,20 +535,20 @@ soup_ssl_get_server_credentials (const char *cert_file, const char *key_file)
 			return NULL;
 	}
 
-	cred = g_new0 (SoupGNUTLSCred, 1);
-	gnutls_certificate_allocate_credentials (&cred->cred);
+	creds = g_new0 (SoupSSLCredentials, 1);
+	gnutls_certificate_allocate_credentials (&creds->creds);
 
-	if (gnutls_certificate_set_x509_key_file (cred->cred,
+	if (gnutls_certificate_set_x509_key_file (creds->creds,
 						  cert_file, key_file,
 						  GNUTLS_X509_FMT_PEM) != 0) {
 		g_warning ("Failed to set SSL certificate and key files "
 			   "(%s, %s).", cert_file, key_file);
-		soup_ssl_free_server_credentials (cred);
+		soup_ssl_free_server_credentials (creds);
 		return NULL;
 	}
 
-	gnutls_certificate_set_dh_params (cred->cred, dh_params);
-	return cred;
+	gnutls_certificate_set_dh_params (creds->creds, dh_params);
+	return creds;
 }
 
 /**
@@ -543,15 +556,13 @@ soup_ssl_get_server_credentials (const char *cert_file, const char *key_file)
  * @creds: a server credentials structure returned by
  * soup_ssl_get_server_credentials().
  *
- * Frees @server_creds.
+ * Frees @creds.
  **/
 void
-soup_ssl_free_server_credentials (gpointer creds)
+soup_ssl_free_server_credentials (SoupSSLCredentials *creds)
 {
-	SoupGNUTLSCred *cred = creds;
-
-	gnutls_certificate_free_credentials (cred->cred);
-	g_free (cred);
+	gnutls_certificate_free_credentials (creds->creds);
+	g_free (creds);
 }
 
 #endif /* HAVE_SSL */
