@@ -19,12 +19,14 @@
 #define mkdir(path, mode) _mkdir (path)
 #endif
 
-gboolean recurse = FALSE;
 SoupSession *session;
 GMainLoop *loop;
+gboolean recurse = FALSE, debug = FALSE;
+const char *method = SOUP_METHOD_GET;
 char *base;
 SoupUri *base_uri;
 int pending;
+GHashTable *fetched_urls;
 
 static GPtrArray *
 find_hrefs (const SoupUri *base, const char *body, int length)
@@ -97,64 +99,10 @@ mkdirs (const char *path)
 	}
 }
 
-static void get_url (const char *url);
-
 static void
-got_url (SoupMessage *msg, gpointer uri)
+print_header (gpointer name, gpointer value, gpointer data)
 {
-	char *name;
-	int fd, i;
-	GPtrArray *hrefs;
-	const char *header;
-
-	name = soup_message_get_uri (msg)->path;
-	if (strncmp (base_uri->path, name, strlen (base_uri->path)) != 0) {
-		fprintf (stderr, "  Error: not under %s\n", base_uri->path);
-		goto DONE;
-	}
-	printf ("%s: %d %s\n", name, msg->status_code, msg->reason_phrase);
-
-	name += strlen (base_uri->path);
-	if (*name == '/')
-		name++;
-
-	if (SOUP_STATUS_IS_REDIRECTION (msg->status_code)) {
-		unlink (name);
-		header = soup_message_get_header (msg->response_headers, "Location");
-		if (header) {
-			printf ("  -> %s\n", header);
-			get_url (header);
-		}
-		goto DONE;
-	}
-
-	if (!SOUP_STATUS_IS_SUCCESSFUL (msg->status_code))
-		goto DONE;
-
-	if (recurse)
-		fd = open (name, O_WRONLY | O_CREAT | O_TRUNC, 0644);
-	else
-		fd = STDOUT_FILENO;
-	write (fd, msg->response.body, msg->response.length);
-	if (!recurse)
-		goto DONE;
-	close (fd);
-
-	header = soup_message_get_header (msg->response_headers, "Content-Type");
-	if (header && g_ascii_strncasecmp (header, "text/html", 9) != 0)
-		goto DONE;
-
-	hrefs = find_hrefs (uri, msg->response.body, msg->response.length);
-	for (i = 0; i < hrefs->len; i++) {
-		get_url (hrefs->pdata[i]);
-		g_free (hrefs->pdata[i]);
-	}
-	g_ptr_array_free (hrefs, TRUE);
-
- DONE:
-	soup_uri_free (uri);
-	if (!--pending)
-		g_main_quit (loop);
+	printf ("%s: %s\n", (const char *)name, (const char *)value);
 }
 
 static void
@@ -162,10 +110,15 @@ get_url (const char *url)
 {
 	char *url_to_get, *slash, *name;
 	SoupMessage *msg;
-	int fd;
+	int fd, i;
+	SoupUri *uri;
+	GPtrArray *hrefs;
+	const char *header;
 
 	if (strncmp (url, base, strlen (base)) != 0)
 		return;
+	if (strchr (url, '?') && strcmp (url, base) != 0)
+		return;
 
 	slash = strrchr (url, '/');
 	if (slash && !slash[1])
@@ -173,6 +126,10 @@ get_url (const char *url)
 	else
 		url_to_get = g_strdup (url);
 
+	if (g_hash_table_lookup (fetched_urls, url_to_get))
+		return;
+	g_hash_table_insert (fetched_urls, url_to_get, url_to_get);
+
 	if (recurse) {
 		/* See if we're already downloading it, and create the
 		 * file if not.
@@ -181,28 +138,83 @@ get_url (const char *url)
 		name = url_to_get + strlen (base);
 		if (*name == '/')
 			name++;
-		if (access (name, F_OK) == 0) {
-			g_free (url_to_get);
+		if (access (name, F_OK) == 0)
 			return;
-		}
 
 		mkdirs (name);
 		fd = open (name, O_WRONLY | O_CREAT | O_TRUNC, 0644);
 		close (fd);
 	}
 
-	msg = soup_message_new (SOUP_METHOD_GET, url_to_get);
+	msg = soup_message_new (method, url_to_get);
 	soup_message_set_flags (msg, SOUP_MESSAGE_NO_REDIRECT);
 
-	pending++;
-	soup_session_queue_message (session, msg, got_url, soup_uri_new (url));
-	g_free (url_to_get);
+	soup_session_send_message (session, msg);
+
+	name = soup_message_get_uri (msg)->path;
+	if (strncmp (base_uri->path, name, strlen (base_uri->path)) != 0) {
+		fprintf (stderr, "  Error: not under %s\n", base_uri->path);
+		return;
+	}
+
+	if (debug) {
+		char *path = soup_uri_to_string (soup_message_get_uri (msg), TRUE);
+		printf ("%s %s HTTP/1.%d\n\n", method, path,
+			soup_message_get_http_version (msg));
+		printf ("HTTP/1.%d %d %s\n",
+			soup_message_get_http_version (msg),
+			msg->status_code, msg->reason_phrase);
+		soup_message_foreach_header (msg->response_headers, print_header, NULL);
+		printf ("\n");
+	} else
+		printf ("%s: %d %s\n", name, msg->status_code, msg->reason_phrase);
+
+	name += strlen (base_uri->path);
+	if (*name == '/')
+		name++;
+
+	if (SOUP_STATUS_IS_REDIRECTION (msg->status_code)) {
+		if (recurse)
+			unlink (name);
+		header = soup_message_get_header (msg->response_headers, "Location");
+		if (header) {
+			if (!debug)
+				printf ("  -> %s\n", header);
+			get_url (header);
+		}
+		return;
+	}
+
+	if (!SOUP_STATUS_IS_SUCCESSFUL (msg->status_code))
+		return;
+
+	if (recurse)
+		fd = open (name, O_WRONLY | O_CREAT | O_TRUNC, 0644);
+	else
+		fd = STDOUT_FILENO;
+	write (fd, msg->response.body, msg->response.length);
+	if (!recurse)
+		return;
+	close (fd);
+
+	header = soup_message_get_header (msg->response_headers, "Content-Type");
+	if (header && g_ascii_strncasecmp (header, "text/html", 9) != 0)
+		return;
+
+	uri = soup_uri_new (url);
+	hrefs = find_hrefs (uri, msg->response.body, msg->response.length);
+	soup_uri_free (uri);
+	for (i = 0; i < hrefs->len; i++) {
+		get_url (hrefs->pdata[i]);
+		g_free (hrefs->pdata[i]);
+	}
+	g_ptr_array_free (hrefs, TRUE);
 }
 
 static void
 usage (void)
 {
-	fprintf (stderr, "Usage: get [-c CAfile] [-p proxy URL] [-r] URL\n");
+	fprintf (stderr, "Usage: get [-c CAfile] [-p proxy URL] [-r] [-h] [-d] URL\n");
 	exit (1);
 }
 
@@ -211,17 +223,27 @@ main (int argc, char **argv)
 {
 	const char *cafile = NULL;
 	SoupUri *proxy = NULL;
+	gboolean synchronous = FALSE;
 	int opt;
 
 	g_type_init ();
 	g_thread_init (NULL);
 
-	while ((opt = getopt (argc, argv, "c:p:r")) != -1) {
+	while ((opt = getopt (argc, argv, "c:dhp:rs")) != -1) {
 		switch (opt) {
 		case 'c':
 			cafile = optarg;
 			break;
 
+		case 'd':
+			debug = TRUE;
+			break;
+
+		case 'h':
+			method = SOUP_METHOD_HEAD;
+			debug = TRUE;
+			break;
+
 		case 'p':
 			proxy = soup_uri_new (optarg);
 			if (!proxy) {
@@ -235,6 +257,10 @@ main (int argc, char **argv)
 			recurse = TRUE;
 			break;
 
+		case 's':
+			synchronous = TRUE;
+			break;
+
 		case '?':
 			usage ();
 			break;
@@ -252,10 +278,19 @@ main (int argc, char **argv)
 		exit (1);
 	}
 
-	session = soup_session_async_new_with_options (
-		SOUP_SESSION_SSL_CA_FILE, cafile,
-		SOUP_SESSION_PROXY_URI, proxy,
-		NULL);
+	fetched_urls = g_hash_table_new (g_str_hash, g_str_equal);
+
+	if (synchronous) {
+		session = soup_session_sync_new_with_options (
+			SOUP_SESSION_SSL_CA_FILE, cafile,
+			SOUP_SESSION_PROXY_URI, proxy,
+			NULL);
+	} else {
+		session = soup_session_async_new_with_options (
+			SOUP_SESSION_SSL_CA_FILE, cafile,
+			SOUP_SESSION_PROXY_URI, proxy,
+			NULL);
+	}
 
 	if (recurse) {
 		char *outdir;
@@ -270,11 +305,13 @@ main (int argc, char **argv)
 		g_free (outdir);
 	}
 
+	if (!synchronous)
+		loop = g_main_loop_new (NULL, TRUE);
+
 	get_url (base);
 
-	loop = g_main_loop_new (NULL, TRUE);
-	g_main_run (loop);
-	g_main_loop_unref (loop);
+	if (!synchronous)
+		g_main_loop_unref (loop);
 
 	soup_uri_free (base_uri);
 
